------------------------------------------------------------------------------
-- user_logic.vhd - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2009 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          user_logic.vhd
-- Version:           1.00.d
-- Description:       User logic.
-- Date:              Mon Aug 23 15:20:32 2010 (by Create and Import Peripheral Wizard)
-- VHDL Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
--   active low signals:                    "*_n"
--   clock signals:                         "clk", "clk_div#", "clk_#x"
--   reset signals:                         "rst", "rst_n"
--   generics:                              "C_*"
--   user defined types:                    "*_TYPE"
--   state machine next state:              "*_ns"
--   state machine current state:           "*_cs"
--   combinatorial signals:                 "*_com"
--   pipelined or register delay signals:   "*_d#"
--   counter signals:                       "*cnt*"
--   clock enable signals:                  "*_ce"
--   internal version of output port:       "*_i"
--   device pins:                           "*_pin"
--   ports:                                 "- Names begin with Uppercase"
--   processes:                             "*_PROCESS"
--   component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v3_00_a;
use proc_common_v3_00_a.proc_common_pkg.all;

library unisim;
use unisim.vcomponents.all;

------------------------------------------------------------------------------
-- Entity section
------------------------------------------------------------------------------
-- Definition of Generics:
--   C_SLV_AWIDTH                 -- Slave interface address bus width
--   C_SLV_DWIDTH                 -- Slave interface data bus width
--   C_NUM_MEM                    -- Number of memory spaces
--
-- Definition of Ports:
--   Bus2IP_Clk                   -- Bus to IP clock
--   Bus2IP_Reset                 -- Bus to IP reset
--   Bus2IP_Addr                  -- Bus to IP address bus
--   Bus2IP_CS                    -- Bus to IP chip select for user logic memory selection
--   Bus2IP_RNW                   -- Bus to IP read/not write
--   Bus2IP_Data                  -- Bus to IP data bus
--   Bus2IP_BE                    -- Bus to IP byte enables
--   IP2Bus_Data                  -- IP to Bus data bus
--   IP2Bus_RdAck                 -- IP to Bus read transfer acknowledgement
--   IP2Bus_WrAck                 -- IP to Bus write transfer acknowledgement
--   IP2Bus_Error                 -- IP to Bus error response
------------------------------------------------------------------------------

entity user_logic is
  generic
  (
    C_SLV_AWIDTH                   : integer              := 32;
    C_SLV_DWIDTH                   : integer              := 32;
    C_NUM_MEM                      : integer              := 1;
    C_MEM0_BASEADDR                : std_logic_vector     := X"F0FF4400";
    C_MEM0_HIGHADDR                : std_logic_vector     := X"F0FF45FF"
  );
  port
  (
    COD_DIN                        : out std_logic;  -- Data Output To CODEC
    COD_LRCIN                      : in  std_logic;  -- I2S LR Signal
    COD_DOUT                       : in  std_logic;  -- Data Input from CODEC
    COD_BCLK                       : in  std_logic;  -- I2S Clock
    SIA_Intr                       : out std_logic;  -- Interrupt

    Bus2IP_Clk                     : in  std_logic;
    Bus2IP_Reset                   : in  std_logic;
    Bus2IP_Addr                    : in  std_logic_vector(0 to C_SLV_AWIDTH-1);
    Bus2IP_CS                      : in  std_logic_vector(0 to C_NUM_MEM-1);
    Bus2IP_RNW                     : in  std_logic;
    Bus2IP_Data                    : in  std_logic_vector(0 to C_SLV_DWIDTH-1);
    Bus2IP_BE                      : in  std_logic_vector(0 to C_SLV_DWIDTH/8-1);
    IP2Bus_Data                    : out std_logic_vector(0 to C_SLV_DWIDTH-1);
    IP2Bus_RdAck                   : out std_logic;
    IP2Bus_WrAck                   : out std_logic;
    IP2Bus_Error                   : out std_logic
  );

  attribute SIGIS : string;
  attribute SIGIS of Bus2IP_Clk    : signal is "CLK";
  attribute SIGIS of Bus2IP_Reset  : signal is "RST";

end entity user_logic;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture imp of user_logic is

component fifo_cod_16x1024
  port (
    rst           : in  std_logic;
    din           : in  std_logic_vector(15 downto 0);
    wr_clk        : in  std_logic;
    wr_en         : in  std_logic;
    dout          : out std_logic_vector(15 downto 0);
    rd_clk        : in  std_logic;
    rd_en         : in  std_logic;
    empty         : out std_logic;
    full          : out std_logic;
    prog_empty    : out std_logic;
    prog_full     : out std_logic;
    rd_data_count : out std_logic_vector(0 downto 0);
    wr_data_count : out std_logic_vector(0 downto 0)
  );
end component;

  signal rec_rst                : std_logic;
  signal rec_fifo_rst           : std_logic;
  signal play_rst               : std_logic;
  signal play_fifo_rst          : std_logic;

  signal rec_dout               : std_logic_vector(0 to 31);
  signal rec_dout_buf_reg       : std_logic_vector(0 to 15);
  signal rec_dout_l             : std_logic_vector(0 to 15);
  signal rec_dout_r             : std_logic_vector(0 to 15);
  signal rec_fifo_rd_en         : std_logic;
  signal rec_wr_en_l            : std_logic;
  signal rec_wr_en_r            : std_logic;
  signal rec_wr_en_l_w          : std_logic;
  signal rec_wr_en_r_w          : std_logic;
  signal rec_fifo_valid         : std_logic;
  signal rec_empty_l            : std_logic;
  signal rec_empty_r            : std_logic;
  signal rec_fifo_hfull         : std_logic;
  signal rec_half_full_l        : std_logic;
  signal rec_half_full_r        : std_logic;
  signal rec_fifo_full          : std_logic;
  signal rec_full_l             : std_logic;
  signal rec_full_r             : std_logic;

  signal play_fifo_din          : std_logic_vector(31 downto 0);
  signal play_dout_l            : std_logic_vector(15 downto 0);
  signal play_dout_r            : std_logic_vector(15 downto 0);
  signal play_fifo_wr_en        : std_logic;
  signal play_fifo_emp          : std_logic;
  signal play_empty_l           : std_logic;
  signal play_empty_r           : std_logic;
  signal play_rd_en_l           : std_logic;
  signal play_rd_en_r           : std_logic;
  signal play_fifo_full         : std_logic;
  signal play_full_l            : std_logic;
  signal play_full_r            : std_logic;
  signal play_fifo_hemp         : std_logic;
  signal play_half_empty_l      : std_logic;
  signal play_half_empty_r      : std_logic;

  signal rec_fifo_valid_intr_en : std_logic;
  signal rec_fifo_hfull_intr_en : std_logic;
  signal play_fifo_hemp_intr_en : std_logic;
  signal play_fifo_full_intr_en : std_logic;

  signal rd_data_count_l        : std_logic_vector(0 downto 0);
  signal wr_data_count_l        : std_logic_vector(0 downto 0);
  signal rd_data_count_r        : std_logic_vector(0 downto 0);
  signal wr_data_count_r        : std_logic_vector(0 downto 0);

  signal mono_sel               : std_logic;
  signal type_sel               : std_logic_vector(0 to 1);

begin

----------------------------------------------------------------------------
-- register
----------------------------------------------------------------------------
  U_A_REGISTER : entity xps_sia00_v1_00_d.A_REGISTER
    generic map (
      C_SLV_AWIDTH    => C_SLV_AWIDTH,
      C_SLV_DWIDTH    => C_SLV_DWIDTH,
      C_BASEADDR      => C_MEM0_BASEADDR,
      C_HIGHADDR      => C_MEM0_HIGHADDR
    )
    port map (
      Bus2IP_Clk       => Bus2IP_Clk,
      Bus2IP_Reset     => Bus2IP_Reset,
      Bus2IP_Addr      => Bus2IP_Addr,
      Bus2IP_RNW       => Bus2IP_RNW,
      IP2Bus_RdAck     => IP2Bus_RdAck,
      IP2Bus_WrAck     => IP2Bus_WrAck,
      IP2Bus_Error     => IP2Bus_Error,
      Bus2IP_Data      => Bus2IP_Data,
      Bus2IP_CS        => Bus2IP_CS(0),
      IP2Bus_Data      => IP2Bus_Data,
      Bus2IP_BE        => Bus2IP_BE,

      REC_RST         => rec_rst,
      PLAY_RST        => play_rst,
      REC_FIFO_HFULL  => rec_fifo_hfull,
      REC_FIFO_VALID  => rec_fifo_valid,
      PLAY_FIFO_FULL  => play_fifo_full,
      PLAY_FIFO_HEMP  => play_fifo_hemp,
      REC_FIFO_FULL   => rec_fifo_full,
      PLAY_FIFO_EMP   => play_fifo_emp, 
      REC_FIFO_RD_EN  => rec_fifo_rd_en,
      PLAY_FIFO_WR_EN => play_fifo_wr_en,
      REC_FIFO_HFULL_INTR_EN => rec_fifo_hfull_intr_en,
      REC_FIFO_VALID_INTR_EN => rec_fifo_valid_intr_en,
      PLAY_FIFO_FULL_INTR_EN => play_fifo_full_intr_en,
      PLAY_FIFO_HEMP_INTR_EN => play_fifo_hemp_intr_en,
      REC_DOUT        => rec_dout,
      PLAY_FIFO_DIN   => play_fifo_din,
      MONO_SEL        => mono_sel,
      TYPE_SEL        => type_sel
    );

----------------------------------------------------------------------------
-- record
----------------------------------------------------------------------------
  U_REC_CODEC : entity xps_sia00_v1_00_d.REC_CODEC
    port map (
      SYS_RST           => Bus2IP_Reset,
      COD_BCLK          => COD_BCLK,
      COD_LRCIN         => COD_LRCIN,
      COD_DOUT          => COD_DOUT,
      rec_wr_en_l       => rec_wr_en_l_w,
      rec_wr_en_r       => rec_wr_en_r_w,
      rec_dout          => rec_dout_buf_reg,
      rec_full_l        => rec_full_l,
      rec_full_r        => rec_full_r,
      mono_sel          => mono_sel
    );

rec_wr_en_l <= rec_wr_en_r_w when type_sel = "01" else rec_wr_en_l_w;
rec_wr_en_r <= rec_wr_en_l_w when type_sel = "01" else rec_wr_en_r_w;

----------------------------------------------------------------------------
-- rec_fifo
----------------------------------------------------------------------------
  rec_fifo_cod_l : fifo_cod_16x1024
    port map (
      rst       => rec_fifo_rst,
      din       => rec_dout_buf_reg,
      wr_clk    => COD_BCLK,
      wr_en     => rec_wr_en_l,
      dout      => rec_dout_l,
      rd_clk    => Bus2IP_Clk,
      rd_en     => rec_fifo_rd_en,
      empty     => rec_empty_l,
      full      => rec_full_l,
      rd_data_count => rd_data_count_l
    );

  rec_fifo_cod_r : fifo_cod_16x1024
    port map (
      rst       => rec_fifo_rst,
      din       => rec_dout_buf_reg,
      wr_clk    => COD_BCLK,
      wr_en     => rec_wr_en_r,
      dout      => rec_dout_r,
      rd_clk    => Bus2IP_Clk,
      rd_en     => rec_fifo_rd_en,
      empty     => rec_empty_r,
      full      => rec_full_r,
      rd_data_count => rd_data_count_r
    );

----------------------------------------------------------------------------
-- data to register
----------------------------------------------------------------------------
  rec_dout <= rec_dout_l & rec_dout_r;

----------------------------------------------------------------------------
-- play_fifo
----------------------------------------------------------------------------
  play_fifo_cod_l : fifo_cod_16x1024
    port map (
      rst        => play_fifo_rst,
      din        => play_fifo_din(31 downto 16),
      wr_clk     => Bus2IP_Clk,
      wr_en      => play_fifo_wr_en,
      dout       => play_dout_l,
      rd_clk     => COD_BCLK,
      rd_en      => play_rd_en_l,
      empty      => play_empty_l,
      full       => play_full_l,
      wr_data_count => wr_data_count_l
   );

  play_fifo_cod_r : fifo_cod_16x1024
    port map (
      rst        => play_fifo_rst,
      din        => play_fifo_din(15 downto 0),
      wr_clk     => Bus2IP_Clk,
      wr_en      => play_fifo_wr_en,
      dout       => play_dout_r,
      rd_clk     => COD_BCLK,
      rd_en      => play_rd_en_r,
      empty      => play_empty_r,
      full       => play_full_r,
      wr_data_count => wr_data_count_r
   );

----------------------------------------------------------------------------
-- play
----------------------------------------------------------------------------
  U_PLAY_CODEC : entity xps_sia00_v1_00_d.PLAY_CODEC
    port map (
      SYS_RST        => Bus2IP_Reset,
      COD_BCLK       => COD_BCLK,
      COD_DIN        => COD_DIN,
      COD_LRCIN      => COD_LRCIN,
      play_dout_l    => play_dout_l,
      play_dout_r    => play_dout_r,
      play_rd_en_l   => play_rd_en_l,
      play_rd_en_r   => play_rd_en_r,
      play_empty_l   => play_empty_l,
      play_empty_r   => play_empty_r,
      mono_sel       => mono_sel
    );

----------------------------------------------------------------------------
-- fifo_reset
----------------------------------------------------------------------------
  rec_fifo_rst <= Bus2IP_Reset or rec_rst;
  play_fifo_rst <= Bus2IP_Reset or play_rst;

----------------------------------------------------------------------------
-- fifo_status
----------------------------------------------------------------------------
  rec_fifo_hfull <= rd_data_count_l(0) and rd_data_count_r(0);
  rec_fifo_valid <= not rec_empty_l and not rec_empty_r;
  play_fifo_full <= play_full_l or play_full_r;
  play_fifo_hemp <= not wr_data_count_l(0) and not wr_data_count_r(0);

  rec_fifo_full  <= rec_full_l or rec_full_r;
  play_fifo_emp  <= play_empty_l or play_empty_r;

----------------------------------------------------------------------------
-- Interrupt
----------------------------------------------------------------------------
  SIA_Intr <= (rec_fifo_hfull and rec_fifo_hfull_intr_en)
           or (rec_fifo_valid and rec_fifo_valid_intr_en)
           or (play_fifo_full and play_fifo_full_intr_en)
           or (play_fifo_hemp and play_fifo_hemp_intr_en);
end imp;
